/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/


package org.jboss.portlet.weather;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletMode;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.net.URL;

public class WeatherPortlet extends GenericPortlet
{
   /**
    * Logger
    */
   public static Logger log = Logger.getLogger(WeatherPortlet.class);

   /**
    * Edit page
    */
   private static final String JSP_EDIT = "/WEB-INF/jsp/edit.jsp";

   /**
    * Default zip set in init param.
    */
   private String DEFAULT_ZIP;

   /**
    * RSS 2.0 XSL page for output
    */
   private static final String RSS_XSL = "/WEB-INF/Rss.xsl";

   /**
    * Compiled Template for output
    */
   private Templates template_2_0;

   /**
    * Base URL of Weather RSS
    */
   private String RSS_URL_PREFIX;

   private static final String E_XSL_UNREADABLE = "Unable to prepare XSL files.";
   private static final String E_UNREADABLE = "Unable to read XML Source.";
   private static final String E_UNPARSEABLE = "Unable to parse XML Source.";

   /**
    * Compile the XSL File.
    */
   public void init()
   {
      try
      {
         InputStream inputstream = getPortletContext().getResourceAsStream(WeatherPortlet.RSS_XSL);
         StreamSource xslSource = new StreamSource(inputstream);
         TransformerFactory tFactory = TransformerFactory.newInstance();
         this.template_2_0 = tFactory.newTemplates(xslSource);

         this.DEFAULT_ZIP = getInitParameter("default_zipcode");
         this.RSS_URL_PREFIX = getInitParameter("base_url");
      }
      catch(TransformerConfigurationException tce)
      {
         log.error(E_XSL_UNREADABLE, tce);
      }
   }

   protected void doView(RenderRequest request, RenderResponse response) throws IOException, PortletException
   {
      response.setProperty("expiration-cache", request.getPreferences().getValue("expires", "180"));

      InputStream xmlInputStream = null;
      InputStream xslInputStream = null;

      String newZip = null;
      try
      {
         newZip = request.getParameter("newzip");
         response.setContentType("text/html");

         if(null == newZip)
         {
            newZip = request.getPreferences().getValue("RssXml", RSS_URL_PREFIX + DEFAULT_ZIP);
         }

         xmlInputStream = new URL(newZip).openStream();
         if(xmlInputStream == null)
         {
            log.error(E_UNREADABLE);
            throw new PortletException(E_UNREADABLE);
         }

         // Ready the parsers
         DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
         DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
         Transformer transformer = this.template_2_0.newTransformer();

         Source xmlSource = null;
         try
         {
            Document document = docBuilder.parse(xmlInputStream);
            transformer = this.template_2_0.newTransformer();
            xmlSource = new DOMSource(document);
         }
         catch(TransformerConfigurationException tce)
         {
            log.error(E_UNPARSEABLE, tce);
            throw new PortletException(E_UNPARSEABLE, tce);
         }

         // Transform document
         PrintWriter writer = response.getWriter();
         StreamResult outStream = new StreamResult(writer);
         transformer.transform(xmlSource, outStream);
      }
      catch(Exception e)
      {
         log.error("Fatal Error reading/parsing XML Source.");
         throw new PortletException(e);
      }
      finally // close all streams
      {
         if(xmlInputStream != null)
         {
            xmlInputStream.close();
         }

         if(xslInputStream != null)
         {
            xslInputStream.close();
         }
      }
   }

   /**
    * Allow edit of weather location.
    *
    * @param request
    * @param response
    * @throws IOException
    * @throws PortletException
    */
   protected void doEdit(RenderRequest request, RenderResponse response) throws IOException, PortletException
   {
      response.setContentType("text/html");
      PortletRequestDispatcher dispatcher = getPortletContext().getRequestDispatcher(JSP_EDIT);
      dispatcher.include(request, response);
   }

   /**
    * Process weather location edit.
    *
    * @param request
    * @param response
    * @throws PortletException
    */
   public void processAction(ActionRequest request, ActionResponse response) throws PortletException
   {
      String newZip = request.getParameter("newzip");

      if(null != newZip)
      {
         PortletPreferences prefs = request.getPreferences();
         try
         {
            prefs.setValue("RssXml", RSS_URL_PREFIX + newZip);
            prefs.store();
         }
         catch(Exception e)
         {
            e.printStackTrace();
         }
      }

      // TODO: error checking for URL + NewZIP

      //store as the selected xml so that it shows up selected as displays
      response.setRenderParameter("newzip", RSS_URL_PREFIX + newZip);

      //set the portlet mode back to view
      response.setPortletMode(PortletMode.VIEW);
   }
}